$title	Standardize Data Set Input Routine

$if not setglobal dataset $setglobal dataset "13x8"

*	Default values are set here:

$if not setglobal homogcoal $setglobal homogcoal yes
$if not setglobal sigmadm   $setglobal sigmadm   4
$if not setglobal sigmamm   $setglobal sigmamm   8
$if not setglobal eta_cru   $setglobal eta_cru   0.5
$if not setglobal eta_gas   $setglobal eta_gas   0.5
$if not setglobal eta_col   $setglobal eta_col   0.5
$if not setglobal bprice    $setglobal bprice    500
$if not setglobal baseline  $setglobal baseline  ref

set	yr		Years covered by the model /1995*2030/,
	tt(yr)		/1995,2000,2005,2010,2015,2020,2025,2030 /,
	t(tt)		Time periods in the calculation 
			/ 2000,2005,2010,2015,2020,2025,2030 /,
	tfirst(t)	First period of the model /2000/,
	tlast(t)	Final period of the model /2030/
	decade(tt)      Decade identifiers /2000, 2010, 2020,2030/
	td(t)		Time periods for plotting / 2000, 2010, 2020, 2030/;

parameter year(yr);  year(yr) = 1995 + (ord(yr)-1);

Display tt, yr, decade, t ; 

*	======================================================================
*	Read the dataset and define regional affiliations and baseline
*	growth rates:

$batinclude mrtdata

$set homcol " "
$if %homogcoal%==yes $set homcol col

*	======================================================================
*	Set definitions specific to the EMF models:

set     nr(i,r)         Non-exhaustible energy production
        ele(i)          Electricity				/ele/,
        xe(i)           Exhaustible energy			/cru,gas,col/
        cru(i)          Crude oil				/cru/,
	oil(i)		Refined oil				/oil/,
        gas(i)          Natural gas				/gas/,
        coal(i)         Coal					/col/,
        nele(i)		Non-electric energy inputs		/gas,oil,col/,
        irep(i)		Sectors to report			/ele/,
        wmkt(i)         Homogeneous energy goods		/cru
								%homcol%/
        ymkt(i,r)       Sectors with output markets
        ttax(r)         Regions with tradable carbon tax
        rtax(r)         Regions with regional carbon tax
	fe(i)           Final energy            /oil, gas, col/
	e(i)		Energy markets
	     /  gas     Natural gas works
                ele     Electricity and heat 
                oil     Refined oil products
                col     Coal transformation
                cru     Crude oil /

	whe(i)  Energy at the wellhead level  /ELE, CRU, GAS, COL/;	

nr(i,r) = yes$(vom(i,r)$(not xe(i)));
ttax(r) = no;
rtax(r) = no;

*	======================================================================
*	Scalars and other parameter definitions:

*	Define scalars characterizing the baseline growth
*	along a uniform steady-state path:

scalar
	time_int	Time interval -- Number of years in each period	/5/
	depr		Annual depreciation rate			/0.07/
	g0		Benchmark growth rate				/0.03/
	invlag		Investment lag (years)				/2/
	r0		Benchmark interest rate				/0.05/,
	pk0		Benchmark capital price
	rk0		Benchmark rental price 
	srvshr		Single period survival share
	termcap		Terminal period capital
	theta		Next period investment maturation share;

*	======================================================================
*	Additional parameters used for model specification:

parameter       
	cecphys(*)      carbon emission coefficients for physical units 
                                /col 0.024, gas 0.0137, oil 0.0181 /

	qref(t)		Reference quantity path
	pref(t)		Reference path of present value prices

        a0(i,r)         Armington supply
        m0(i,r)         Aggregate import supply
        d0(i,r)         Aggregate domestic supply

        c0(i,r)         Final demand by commodity
        tc(i,r)         Tax rate on final demand
        ct0(r)          Total final demand
        pc0(i,r)        Reference price for final demand

        ld0(i,r)        Base year labor demand by sector
        kd0(i,r)        Base year capital earnings by sector
        
        pmx0(i,s,r)     Reference price of imports
        pmt0(i,s,r)     Reference price of transport services

        aeei(i,tt,r)		Energy efficiency index
	aeeirate(i,tt,r)	Rate of autonomous energy efficiency improvement (%)
	growthrate		Baseline growth rates for GDP and carbon (%)

	bp(t)		Backstop price level -- dollars per tonne

        tmoil(r)        Crude oil import tariff
        txoil(r)        Crude oil export tariff

        tshare(i,r)     Transport cost share
        ex0(i,r)        Homogeneous exports
        em0(i,r)        Homogeneous imports
        et0(i,r)        Transport cost on homogeneous imports
        txe(i,r)        Export tax rate
        tme(i,r)        Import tax rate
        vex0(i,r)       Value of homogeneous exports
        vem0(i,r)       Value of homogeneous imports
        mktadj(i,r)     Market adjustment accomodating homogeneous trade

        rd0(i,r)        Resource inputs for fossil fuel production
        prref(i,t,r)	Reference price of energy-supply resource

        ffsupply(t,i,r) Fossil fuel supply

        carbcoef(j,*,r) Carbon coefficients
        carblim(r,t)    Carbon emission limit
        mkup(t,r)       Mark up for CRTS

        isub(t,r)       Intra-fuel substitution in final demand
        esub(*,t,R)     Elasticity between VA and energy
        e_vae(r)        Elasticity of substitution in the VA production nest
        e_elec(t,r)     Elasticity between electric and non-electric energy
        e_c_go(i,t,r)   Elasticity between coal and the oil-gas aggregate 
        e_g_o(i,t,r)    Elasticity of substitution between oil and gas in production

        esub_es(i,t,r)  Elasticity of substitution in energy production
        thetar(i,t,r)   Value share of resource inputs in fossil supply

        eta(r,xe)       Elasticity of supply 

        esubdm(i,r)     Armington elasticity - domestic vesus imports
        esubmm(i,r)     Armington elasticity - intra-imports

        gdp(tt,r)       GDP index for future projection

	md(i,t,s,r)	Import demand index
	dd(i,t,r)	Domestic demand index
	
        pytarget        Target baseline prices for oil - coal - gas,

	carbon95(r,*)	1995 carbon emissions (IEA)

	xt(wmkt,t,r)    Export tax for fossil fuels,

        ccoef(r,i)	Carbon coefficients
        vshare(r,i)	Energy value shares for non-energy sectors
        carbinv(*,*)	CO2 inventories (IEA)-- mton
        carbint(r,i)	Sectoral CO2 intensities --kg per $output 

	ffs		Fossil fuel production
	growth		GDP growth comparison

	energy		Energy use

	pfe0(i,i,t,r)	Baseline reference price of energy
	backprc(t,r)	Backstop prices	

	pvtax(t,r)	Present value of tax revenue
	energyinp	Energy sector inputs (echo print)

	gr0(r)		Benchmark growth rates
	i0ratio(r)	Ratio of benchmark investment to capital;


*	======================================================================
*	Adopt some notation which is different from the core GTAPinGAMS model:

a0(i,r) = vipm(i,r) + vdpm(i,r) + vigm(i,r) + vdgm(i,r) + vifm(i,r) + vdfm(i,r);
d0(i,r) = vdpm(i,r) + vdgm(i,r) + vdfm(i,r);
m0(i,r) = vipm(i,r) + vigm(i,r) + vifm(i,r);

c0(i,r) = vpm(i,r) + vgm(i,r);
tc(i,r)$c0(i,r) = (tp(i,r)*vpm(i,r) + tg(i,r)*vgm(i,r)) / c0(i,r);
ct0(r) = sum(i, c0(i,r)*(1+tc(i,r)));
pc0(i,r) = 1 + tc(i,r);
ct0(r) = sum(i, pc0(i,r)*c0(i,r));
vg(r) = 0;

ld0(i,r) = vfm("lab",i,r);
kd0(i,r) = vfm("cap",i,r);
evoa("lab",r) = sum(i, ld0(i,r));
evoa("cap",r) = sum(i, kd0(i,r));
        
pmx0(i,s,r) = (1 + tm(i,s,r))*(1+tx(i,s,r));
pmt0(i,s,r) = 1 + tm(i,s,r);

*	======================================================================
*       Assign elasticities of substitution:

isub(t,r)     = 1.0;

* Make esub time varying.  Linear interpolation between esub_short and
* esub_long.  In 2030, esub = esub_long

parameter
	esub_short(*) value of esub in 2000 
	/USA 0.35, CAN 0.35, EUR 0.35, JPN 0.35, OOE 0.35, 
	 ROW 0.30, BRA 0.30, CHN 0.30, FSU 0.30, IDN 0.30, 
	 KOR 0.30, IDI 0.30, EAS 0.30, OAS 0.30, SEA 0.30, CHI 0.30,
	 ASI 0.30, MPC 0.30, CSA 0.30, SSA 0.30, TIG 0.30/							  

	esub_long(*)  value of esub in 2030 
	/USA 0.6, CAN 0.6, EUR 0.6, JPN 0.6, OOE 0.6, 
	 ROW 0.5, BRA 0.5, CHN 0.5, FSU 0.6, IDN 0.5, 
	 KOR 0.5, IDI 0.5, EAS 0.5, OAS 0.5, SEA 0.5, CHI 0.5,
	 ASI 0.5, MPC 0.5, CSA 0.5, SSA 0.5, TIG 0.5/ ;

*	Adjust short-run elasticities:

scalar esub_adj /0/;

$if %esub%==lo esub_adj = -0.1;
$if %esub%==hi esub_adj =  0.1;

esub_short(r) = esub_short(r) + esub_adj;

display esub_short;

*	Randomly generate the elasticities:

$if setglobal seed option seed=%seed%;
$if setglobal seed esub_short(r) = uniform(0.5,1.5) * esub_short(r);
$if setglobal seed esub_long(r)  = uniform(0.5,1.5) * esub_long(r);

*--  Code for time varying elasticities --*

esub(i,t,r)$(NOT ELE(i)) = esub_short(r) * (2030 - year(t)) / 30 
			    + esub_long(r) * (year(t) - 2000) / 30;
esub("final",t,r) = esub_short(r) * (2030 - year(t)) / 30 
			+ esub_long(r) * (year(t) - 2000) / 30;

Loop(t$(year(t) gt 2030),
	esub(i,t,r)$(NOT ELE(i)) = esub(i,"2030",r);
	esub("final",t,r)= esub("final","2030",r); 
);

display esub;

e_vae(r)      = 1.0;
e_elec(t,r)   = 0.1;

scalar t_elas	Start of time varying isub /2010/ ;

e_c_go(i,t,r)	  = 0.5 ; 
e_g_o(i,t,r)	  = 1.0 ; 
e_c_go("ELE",t,r) = 1 ;
e_g_o("ELE",t,r)  = 2 ;
e_c_go("ELE",t,r)$(year(t) ge t_elas)   = 1.0 * (2030 - year(t)) / (2030 - t_elas) 
					+ 2.0 * (-t_elas + year(t)) / (2030 - t_elas) ;
e_g_o("ELE",t,r)$(year(t) ge t_elas)  = 2.0 * (2030 - year(t)) / (2030 - t_elas) 
				      + 4.0 * (-t_elas + year(t)) / (2030 - t_elas) ;
isub(t,r)$(year(t) ge t_elas)	= 2.0 * (2030 - year(t)) / (2030 - t_elas) 
				+ 4.0 * (-t_elas + year(t)) / (2030 - t_elas) ;
loop(t$(year(t) GT 2030),
	e_c_go("ELE",t,r) = E_c_go("ELE","2030",r); 
	e_g_o("ELE",t,r)= e_g_o("ELE","2030",r);
	isub(t,r)= isub("2030",r); );
display e_elec, e_c_go, e_g_o, isub;

*	Elasticities of fossil fuel supply:

eta(r,"cru")  = %eta_cru%;
eta(r,"gas")  = %eta_gas%;
eta(r,"col")  = %eta_col%;

esubdm(i,r)  = %sigmadm%;
esubmm(i,r)  = %sigmamm%;

*	Randomly generated elasticities are one option:

$if setglobal seed eta(r,"cru") = eta(r,"cru") * uniform(0.25, 2);
$if setglobal seed eta(r,"col") = eta(r,"col") * uniform(0.25, 2);
$if setglobal seed esubdm(i,r) = %sigmadm% * uniform(0.25, 2);
$if setglobal seed esubmm(i,r) = 2 * esubdm(i,r);

esubdm(fe,r) = 0.5;

*	======================================================================
*       Establish resource inputs for resource supply:

rd0(xe,r) = kd0(xe,r);
kd0(xe,r) = 0;


*	Allocate part of the rd0 to labor to see if it affects esub_es(GLY)

rd0(xe,r) = 0.75*rd0(xe,r);
ld0(xe,r) = ld0(xe,r) + (1/3)*rd0(xe,r);

*	Recalibrate labor and capital demand:

vfm("lab",i,r) = ld0(i,r);
vfm("cap",i,r) = kd0(i,r);

evoa("lab",r) = sum(i, ld0(i,r));
evoa("cap",r) = sum(i, kd0(i,r));

thetar(xe,t,r)$VOM(xe,R) = rd0(xe,r) / (vom(xe,r) * (1-ty(xe,r)));

esub_es(xe,T,R)$(1-thetar(xe,t,r)) = thetar(xe,t,r) * eta(r,xe) / (1-thetar(xe,t,r));

display  thetar, esub_es;

esub_es(xe,t,r) = 0;
prref(xe,t,r) = 1;

*	======================================================================
*       Convert GTAP bilateral trade flows for homogeneous goods
*	(crude oil and possibly coal):

vdm(wmkt,r) = vdm(wmkt,r) + vxm(wmkt,r);
vxm(wmkt,r) = 0;
vim(wmkt,r) = 0;
m0(wmkt,r)  = 0;

em0(wmkt,r) = max(0, +sum(s, vxmd(wmkt,s,r)*(1+tx(wmkt,s,r)))
                     -sum(s, vxmd(wmkt,r,s)*(1+tx(wmkt,r,s))));
ex0(wmkt,r) = max(0, -sum(s, vxmd(wmkt,s,r)*(1+tx(wmkt,s,r)))
                     +sum(s, vxmd(wmkt,r,s)*(1+tx(wmkt,r,s))));

txe(wmkt,r)$ex0(wmkt,r) = round( sum(s, vxmd(wmkt,r,s)*tx(wmkt,r,s)) /
                                 sum(s, vxmd(wmkt,r,s)), 2);

vex0(wmkt,r) = ex0(wmkt,r) / (1 + txe(wmkt,r));

tshare(wmkt,r)$em0(wmkt,r) = sum(s, vtwr(wmkt,s,r)) /
                sum((s,rr)$em0(wmkt,rr), vtwr(wmkt,s,rr));

et0(wmkt,r)$em0(wmkt,r) = tshare(wmkt,r) * sum((s,rr), vtwr(wmkt,s,rr));

tme(wmkt,r)$em0(wmkt,r) = round(
          sum(s, (vxmd(wmkt,s,r)*(1+tx(wmkt,s,r))+vtwr(wmkt,s,r))*tm(wmkt,s,r)) /
          sum(s, (vxmd(wmkt,s,r)*(1+tx(wmkt,s,r))+vtwr(wmkt,s,r)) ), 2);

vem0(wmkt,r) = (em0(wmkt,r) + et0(wmkt,r)) * (1 + tme(wmkt,r));

tx(wmkt,s,r) = 0;
tm(wmkt,s,r) = 0;

*	We need to have a small market adjustment for this conversion:

mktadj(wmkt,r)$a0(wmkt,r) = (a0(wmkt,r) - vdm(wmkt,r) - vem0(wmkt,r) + vex0(wmkt,r))
                                / a0(wmkt,r);
display mktadj;

*       Finally, convert all inputs into the Armington nest into the py market:

d0(wmkt,r) = a0(wmkt,r);

*       Recompute capital flows in order to account for changes in 
*       crude oil trade flows etc:

vb(r) = ct0(r) + vi(r)
        - sum(f, evoa(f,r)) - sum(i, rd0(i,r))
	- sum(wmkt, mktadj(wmkt,r)*a0(wmkt,r))
	- sum(wmkt, tme(wmkt,r) * (em0(wmkt,r) + et0(wmkt,r)))
	- sum(wmkt, txe(wmkt,r) * vex0(wmkt,r))
        - sum(i,     ty(i,r)   * vom(i,r))
        - sum((i,j), ti(j,i,r) * vafm(j,i,r))
        - sum((i,f), tf(f,i,r) * vfm(f,i,r))
        - sum((i,s), tx(i,r,s)  * vxmd(i,r,s))
        - sum((i,s), tm(i,s,r)  * (vxmd(i,s,r)*(1+tx(i,s,r)) + vtwr(i,s,r)) )
        - sum(i,     tc(i,r)*c0(i,r));
vb("checksum") = sum(r, vb(r));
display vb;

*	======================================================================
*	Generate base year carbon emissions by sector and then echo-print some
*	benchmark statistics:

carbcoef(fe,"final",r) = cecphys(fe) * efd(fe,r);
carbcoef(fe,i,r)       = cecphys(fe) * eind(fe,i,r);

carbon95(r,fe) = carbcoef(fe,"final",r) + sum(i, carbcoef(fe,i,r));
carbon95(r,"total") = sum(fe, carbon95(r,fe));

*	Generate a report of energy use in physical units:

energy(e,r,i)       = round(eind(e,i,r), 1);
energy(e,r,"final") = round(efd(e,r), 1);
option energy:1:1:1;
display energy;

*	Energy value shares in non-energy production:

vshare(r,i)$(vom(i,r) and (not e(i))) 
        = round( 100 * sum(e, vafm(e,i,r)*(1+ti(e,i,r))) / vom(i,r), 1);

ccoef(r,nele) = cecphys(nele);

carbinv(r,"ind_nele")=1000 * sum((e,i)$(not ele(i)), ccoef(r,e)*eind(e,i,r));
carbinv(r,"fd_nele")= 1000 * sum(e, ccoef(r,e)*efd(e,r));

ccoef(r,ele) = sum(nele, cecphys(nele) * eind(nele,ele,r))/
                        (sum(i, eind(ele,i,r)) + efd(ele,r));

*	Carbon inventories:

carbinv(r,"electric")=1000*sum((e,ele), ccoef(r,e)*eind(e,ele,r));
carbinv(r,"total")=1000*sum(e, ccoef(r,e)*(sum(i$(not ele(i)), eind(e,i,r))+efd(e,r)) );
carbinv(r,"ind_total")=1000 * sum((e,i)$(not ele(i)), ccoef(r,e)*eind(e,i,r));
carbinv(r,"fd_total")= 1000 * sum(e, ccoef(r,e)*efd(e,r));
carbinv(r,"kg/$")= round(carbinv(r,"total")/(10*sum((f,i), vfm(f,i,r))), 1);

set jrep /total,ind_total, fd_total, ind_nele, fd_nele, electric/;
carbinv("total",jrep) = sum(r, carbinv(r,jrep));

*	Generate reports for carbon intensities in electricity
*	and non-energy sectors:

irep(i)$(not e(i)) = yes;
irep(ele) = yes;

carbint(r,ele)$vom(ele,r) = 
  round(100*sum(nele, cecphys(nele) * eind(nele,ele,r))/vom(ele,r),1);
carbint(r,irep)$vom(irep,r) = 
  round(100*sum(e, ccoef(r,e)*eind(e,irep,r))/vom(irep,r), 1);

option vshare:1, carbinv:1, carbint:1;
display vshare, carbinv, carbint;

*	======================================================================
*       Identify markets which should be included in the model,
*	including those for crude oil which may be imported but not produced:

ymkt(i,r) = yes$vom(i,r);
ymkt(wmkt,r)$em0(wmkt,r) = yes;

*	======================================================================
*	Calibration of savings and investment to a uniform steady-state 
*	equilibrium across all regions:

srvshr  = (1-depr)**time_int;
theta   = invlag / time_int;
pk0     = (1/time_int) / (1/(1 + r0)**time_int * theta + (1-theta));

energyinp(e,j,r)$vom(e,r)   = vafm(j,e,r)/vom(e,r);
energyinp(e,"l",r)$vom(e,r) = ld0(e,r)/vom(e,r);
energyinp(e,"k",r)$vom(e,r) = kd0(e,r)/vom(e,r);
energyinp(e,"r",r)$vom(e,r) = rd0(e,r)/vom(e,r);
display energyinp;

parameter kvs(r)	aggregate capital value share 
	  vak(r)	value added -- capital
	  i0targ(r)	investment target
	  k0(r)		initial capital stock
	  capital(r,*)	Capital stock statistics
	  inv_adj(R)	Required percentage adjustment in investment
	  thetak(*,r)	base year capital stock share
	  cgdprofit	Check on profits in investment,
	  va0(i,r)	Base year value-added;

rk0     = (1-srvshr/(1+r0)**time_int) * pk0;
kvs(r) = sum(i, kd0(i,r)) / sum(i, ld0(i,r) + kd0(i,r));
vak(r) = sum(i, kd0(i,r));
k0(r)  = vak(r)/rk0;
capital(r,"valueshare") = rk0*k0(r)/sum(i, ld0(i,r) + kd0(i,r)) ;
capital(r,"ratio") = k0(r)/sum(i, ld0(i,r) + kd0(i,r)) ;
display  capital, rk0, pk0, srvshr;

thetak(i,r) = (kd0(i,r)/rk0) / k0(r);


*	Energy price target:

pfe0(fe,i,t,r) = pai0(fe,i,r);

*	Backstop prices:

bp(t) = %bprice%;



variables	vcd(i,r)	calibrated value of consumption
		vid(i,r)	calibrated value of investment demand
		obj		least squares objective;

positive variables vcd, vid;

equations	vdbal(i,r)	demand balance
		vibal(r)	investment valance
		objdef		defines the objective;

objdef..	obj =e= sum((i,r)$c0(i,r), sqr(vcd(i,r)-c0(i,r))/c0(i,r))
		+ sum((i,r)$vafm(i,"cgd",r), sqr(vid(i,r)-vafm(i,"cgd",r))/vafm(i,"cgd",r));

vibal(r)..	i0targ(r) * (1-ty("cgd",r)) =e= ld0("cgd",r)+kd0("cgd",r) 
			+ sum(i, vid(i,r)*pai0(i,"cgd",r));

vdbal(i,r)..	vid(i,r) + vcd(i,r) =e= vafm(i,"cgd",r) + c0(i,r);

model dbal /objdef, vdbal, vibal/;

*	Recalibrate investment:


scalar uniformg Switch for uniform growth rate /1/;

$include recalib


